﻿namespace BigWhiteBall.Utility
{
  using System;
  using System.Collections.Generic;
  using Windows.Devices.Gpio;

  enum RotaryEncoderTurnDirection
  {
    Clockwise,
    Anticlockwise
  }
  class RotaryEncoderTurnEventArgs
  {
    public RotaryEncoderTurnEventArgs(RotaryEncoderTurnDirection direction)
    {
      this.Direction = direction;
    }
    public RotaryEncoderTurnDirection Direction { get; }
  }
  class RotaryEncoderWithSwitch
  {
    public event EventHandler<RotaryEncoderTurnEventArgs> Turn;
    public event EventHandler SwitchPressed;

    public RotaryEncoderWithSwitch(int aDtPin, int bClkPin, int switchPin)
    {
      this.aDtPin = aDtPin;
      this.bClkPin = bClkPin;
      this.switchPin = switchPin;
      this.direction = RotaryEncoderTurnDirection.Clockwise;

      this.pins = new Dictionary<int, GpioPin>();

      foreach (var pin in new int[] { this.aDtPin, this.bClkPin, this.switchPin })
      {
        GpioPin gpioPin = GpioController.GetDefault().OpenPin(pin);
        gpioPin.ValueChanged += OnValueChanged;
        this.pins[pin] = gpioPin;

        if (pin == this.switchPin)
        {
          gpioPin.Write(GpioPinValue.High);
        }
        gpioPin.SetDriveMode(GpioPinDriveMode.Input);

      }
    }
    void OnValueChanged(GpioPin sender, GpioPinValueChangedEventArgs args)
    {
      if (sender == this.pins[this.switchPin])
      {
        if (args.Edge == GpioPinEdge.FallingEdge)
        {
          var handlers = this.SwitchPressed;
          if (handlers != null)
          {
            handlers(this, EventArgs.Empty);
          }
        }
      }
      else
      {
        var aPinValue = this.pins[this.aDtPin].Read();
        var bPinValue = this.pins[this.bClkPin].Read();

        var highCount =
          ((aPinValue == GpioPinValue.High) ? 1 : 0) +
          ((bPinValue == GpioPinValue.High) ? 1 : 0);

        if (highCount == (previousHighCount + 1))
        {
          if (previousHighCount == 0)
          {
            this.direction = aPinValue == GpioPinValue.High ?
              RotaryEncoderTurnDirection.Anticlockwise : RotaryEncoderTurnDirection.Clockwise;
          }
          previousHighCount = highCount;
        }
        else if ((previousHighCount == 2) && (highCount == 1))
        {
          // we're on the falling edge. we flag this as a rotation.
          var handlers = this.Turn;

          if (handlers != null)
          {
            handlers(this, new RotaryEncoderTurnEventArgs(this.direction));
          }
          previousHighCount = 0;
        }
        else
        {
          // start over, accepting that we might miss some.
          previousHighCount = 0;
        }
      }
    }
    Dictionary<int, GpioPin> pins;
    int aDtPin;
    int bClkPin;
    int switchPin;
    int previousHighCount;
    RotaryEncoderTurnDirection direction;
  }
}
