﻿namespace BigWhiteBall.ViewModels
{
  using System;
  using BigWhiteBall.Services;
  using Utility;
  using Windows.System.Profile;
  using System.Threading;

  class DriveSpheroControlViewModel : ViewModelBase
  {
    public DriveSpheroControlViewModel(ISpheroService spheroService)
    {
      this.spheroService = spheroService;
      this.context = SynchronizationContext.Current;

      // New.
      if (this.IsIoTFamily)
      {
        this.rotaryEncoder = new RotaryEncoderWithSwitch(
          A_PIN_DT, B_PIN_CLK, SW_PIN);

        this.rotaryEncoder.Turn += OnGpioTurnInput;
        this.rotaryEncoder.SwitchPressed += OnGpioSwitchPressed;
      }
    }
    public float SpeedValue
    {
      get
      {
        return (this.speedValue);
      }
      set
      {
        base.SetProperty(ref this.speedValue, value);
        this.DriveSphero();
      }
    }
    public double RotationValue
    {
      get
      {
        return (this.rotationValue);
      }
      set
      {
        base.SetProperty(ref this.rotationValue, value);
        this.RotateSphero();
      }
    }
    void DriveSphero()
    {
      this.spheroService.Drive(this.speedValue / MAX_SLIDER_VALUE);
    }
    void RotateSphero()
    {
      this.spheroService.Rotate((int)this.rotationValue);
    }
    /// <summary>
    /// New
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    void OnGpioSwitchPressed(object sender, EventArgs e)
    {
      this.context.Post(
        _ =>
        {
          var newSpeed = this.SpeedValue + SPEED_INCREMENT;

          // We have two speeds - half way and stopped!
          if (newSpeed >= MAX_SLIDER_VALUE)
          {
            newSpeed = 0.0f;
          }
          this.SpeedValue = newSpeed;
        },
        null
      );
    }
    /// <summary>
    /// New
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    void OnGpioTurnInput(object sender, RotaryEncoderTurnEventArgs e)
    {
      this.context.Post(
        _ =>
        {
          var delta =
            (e.Direction == RotaryEncoderTurnDirection.Clockwise) ?
            ANGLE_ROTATION : 0 - ANGLE_ROTATION;

          var newAngle = this.RotationValue + delta;

          if ((newAngle > 360.0d) || (newAngle <= 0.0d))
          {
            newAngle = 0.0d;
          }
          this.RotationValue = newAngle;
        },
        null);
    }
    /// <summary>
    /// New
    /// </summary>
    bool IsIoTFamily
    {
      get
      {
        // NB. I can use contract detection and type detection and method detection
        // but the problem is that the Windows.Devices.DevicesLowLevelContract says
        // that it is available on the PC and the GpioController type says that
        // it is available on the PC but when you try and get the default controller
        // the method throws and so doing all that contract detection becomes a
        // waste of time in that case.
        return (
          AnalyticsInfo.VersionInfo.DeviceFamily == IOT_FAMILY);
      }
    }
    float speedValue;
    static readonly float MAX_SLIDER_VALUE = 100.0f;
    double rotationValue;
    ISpheroService spheroService;

    // New: Added the RotaryEncoderWithSwitch for IoT platforms.
    RotaryEncoderWithSwitch rotaryEncoder;

    // New 
    SynchronizationContext context;

    // New: Added these hard coded pin values.
    static readonly int A_PIN_DT = 5;
    static readonly int B_PIN_CLK = 6;
    static readonly int SW_PIN = 22;

    // New
    static readonly string IOT_FAMILY = "Windows.IoT";

    // New
    static readonly double ANGLE_ROTATION = 20.0d;
    static readonly int SPEED_INCREMENT = 50;
  }
}
